package event

import "encoding/xml"

// InfoType 说明
const (
	// INFO_TYPE_COMPONENT_VERIFY_TICKET 验证票据
	INFO_TYPE_COMPONENT_VERIFY_TICKET = "component_verify_ticket"

	// INFO_TYPE_AUTHORIZED 授权成功
	INFO_TYPE_AUTHORIZED = "authorized"

	// INFO_TYPE_UPDATEAUTHORIZED 更新授权
	INFO_TYPE_UPDATEAUTHORIZED = "updateauthorized"

	// INFO_TYPE_UNAUTHORIZED 取消授权
	INFO_TYPE_UNAUTHORIZED = "unauthorized"
)

// CDATA 解析 xml cdata 字段
type CDATA struct {
	Value string `xml:",cdata"`
}

// ReceivedEncryptMessage 待解密数据
type ReceivedEncryptMessage struct {
	XMLName    xml.Name `xml:"xml"`
	ToUserName string   `xml:"ToUserName"`
	Encrypt    string   `xml:"Encrypt"`
}

type SendEncryptMessage struct {
	XMLName      xml.Name `xml:"xml"`
	Encrypt      CDATA    `xml:"Encrypt"`
	MsgSignature string   `xml:"MsgSignature"`
	TimeStamp    int64    `xml:"TimeStamp"`
	Nonce        string   `xml:"Nonce"`
}

// AuthEvent 事件
type AuthEvent struct {
	XMLName    xml.Name `xml:"xml"`
	AppID      string   `xml:"AppId"`
	CreateTime int64    `xml:"CreateTime"`
	InfoType   string   `xml:"InfoType"`
}

// ComponentVerifyTicketEvent 令牌授权事件
type ComponentVerifyTicketEvent struct {
	AuthEvent
	ComponentVerifyTicket string `xml:"ComponentVerifyTicket"`
}

// AuthorizedEvent 授权成功, 与更新授权字段属性一致
type AuthorizedEvent struct {
	AuthEvent
	AuthorizerAppID              string `xml:"AuthorizerAppid"`
	AuthorizationCode            string `xml:"AuthorizationCode"`
	AuthorizationCodeExpiredTime int64  `xml:"AuthorizationCodeExpiredTime"`
	PreAuthCode                  string `xml:"PreAuthCode"`
}

// UnauthorizedEvent 取消授权
type UnauthorizedEvent struct {
	AuthEvent
	AuthorizerAppID string `xml:"AuthorizerAppid"`
}

// AuthorizerEvent 公众号或者小程序事件
type AuthorizerEvent struct {
	XMLName      xml.Name `xml:"xml"`
	ToUserName   CDATA    `xml:"ToUserName"`
	FromUserName CDATA    `xml:"FromUserName"`
	CreateTime   int64    `xml:"CreateTime"`
	MsgType      CDATA    `xml:"MsgType"`
	Event        CDATA    `xml:"Event"`
	MsgID        int64    `xml:"MsgId"`
}

// TextMessage 文本消息
type TextMessage struct {
	AuthorizerEvent
	Content CDATA `xml:"Content"`
}

// ImageMessage 图片消息
type ImageMessage struct {
	AuthorizerEvent
	PicURL  CDATA `xml:"PicUrl"`
	MediaID CDATA `xml:"MediaId"`
}

// VoiceMessage 语音消息
type VoiceMessage struct {
	AuthorizerEvent
	MediaID     CDATA `xml:"MediaId"`
	Format      CDATA `xml:"Format"`
	Recognition CDATA `xml:"Recognition"`
}

// VideoMessage 视频消息
type VideoMessage struct {
	AuthorizerEvent
	MediaID      CDATA `xml:"MediaId"`
	ThumbMediaID CDATA `xml:"ThumbMediaId"`
}

// ShortVideoMessage 小视频消息
type ShortVideoMessage struct {
	AuthorizerEvent
	MediaID      CDATA `xml:"MediaId"`
	ThumbMediaID CDATA `xml:"ThumbMediaId"`
}

// LocationMessage 地理位置消息
type LocationMessage struct {
	AuthorizerEvent
	LocationX float64 `xml:"Location_X"`
	LocationY float64 `xml:"Location_Y"`
	Scale     int     `xml:"Scale"`
	Label     CDATA   `xml:"Label"`
}

// LinkMessage 链接消息
type LinkMessage struct {
	AuthorizerEvent
	Title       CDATA `xml:"Title"`
	Description CDATA `xml:"Description"`
	URL         CDATA `xml:"Url"`
}

// SubscribeEvent 关注/取消关注事件
// event subscribe(订阅)、unsubscribe(取消订阅)
type SubscribeEvent struct {
	AuthorizerEvent
}

// QRSceneEvent 扫描带参数二维码事件
type QRSceneEvent struct {
	AuthorizerEvent
	EventKey CDATA `xml:"EventKey"`
	Ticket   CDATA `xml:"Ticket"`
}

// LocationEvent 上报地理位置事件
type LocationEvent struct {
	AuthorizerEvent
	Latitude  float64 `xml:"Latitude"`
	Longitude float64 `xml:"Longitude"`
	Precision float64 `xml:"Precision"`
}

// MenuEvent 自定义菜单事件
// event 值为 CLICK
type MenuEvent struct {
	AuthorizerEvent
	EventKey CDATA `xml:"EventKey"`
}
